%
% MCDCStrategyChoice2.m
%

classdef MCDCStrategyChoice2 < MCDCStrategy

  methods (Access = public)

    function this = MCDCStrategyChoice2(algorithm)
      this = this@MCDCStrategy(algorithm);
      this.particleFilter = @PMMHParticleFilter;
    end

    function result = StateMean(this, j, in, out)
      if j == 1
        gridPoints = this.gridData.GetFlatMatrix();
        dims = [size(in.stateMeanFuncs, 2) this.gridData.CountPerDim];
        this.initialStateMean = zeros(dims);
        for d = 1:dims(1)
          this.initialStateMean(d, :) = in.stateMeanFuncs{d}(gridPoints);
        end
        result = this.initialStateMean;
      else
        result = reshape(out.FVAt(j - 1), size(this.initialStateMean));
      end
    end

    function result = ObsMean(this, j, in, out)
      if j == 1
        gridPoints = this.gridData.GetFlatMatrix();
        dims = [size(in.obsMeanFuncs, 2) this.gridData.CountPerDim];
        this.initialObsMean = zeros(dims);
        for d = 1:dims(1)
          this.initialObsMean(d, :) = in.obsMeanFuncs{d}(gridPoints);
        end
        result = this.initialObsMean;
      else
        result = reshape(out.GVAt(j - 1), size(this.initialObsMean));
      end
    end

    function result = StateSigma(this, j, gramMatrix)
      result = gramMatrix;
    end

    function result = ObsSigma(this, j, gramMatrix)
      result = gramMatrix;
    end

    function result = AcceptanceLogProbability(this, j, in, out, logLikelihood, stateVals, obsVals, gramMatState, gramMatObs)
      if j == 1
        result = 0;
      else
        result = this.LikelihoodContribution(j, out, logLikelihood) ...
               + this.PriorContribution(j, in, out, stateVals, obsVals, gramMatState, gramMatObs) ...
               + this.TransitionContribution(j, in, out, stateVals, obsVals, gramMatState, gramMatObs);
      end
    end

  end

  methods (Access = private)

    function result = LikelihoodContribution(this, j, out, logLikelihood)
      result = logLikelihood - out.loglikAt(j - 1);
    end

    function result = PriorContribution(this, j, in, out, stateVals, obsVals, gramMatState, gramMatObs)
      xdim = size(stateVals, 1);
      ydim = size(obsVals, 1);
      result =          this.algorithm.CalcLogProbability(stateVals                                , this.gridData, in.gridDimForGramMatrix, this.initialStateMean, gramMatState         );
      result = result + this.algorithm.CalcLogProbability(obsVals                                  , this.gridData, in.gridDimForGramMatrix, this.initialObsMean  , gramMatObs           );
      result = result - this.algorithm.CalcLogProbability(reshape(out.FVAt(j - 1), size(stateVals)), this.gridData, in.gridDimForGramMatrix, this.initialStateMean, this.prevGramMatState);
      result = result - this.algorithm.CalcLogProbability(reshape(out.GVAt(j - 1), size(obsVals  )), this.gridData, in.gridDimForGramMatrix, this.initialObsMean  , this.prevGramMatObs  );
    end

    function result = TransitionContribution(this, j, in, out, stateVals, obsVals, gramMatState, gramMatObs)
      xdim = size(stateVals, 1);
      ydim = size(obsVals, 1);
      prevStateVals = reshape(out.FVAt(j - 1), size(stateVals));
      prevObsVals   = reshape(out.GVAt(j - 1), size(obsVals  ));
      result =          this.algorithm.CalcLogProbability(prevStateVals, this.gridData, in.gridDimForGramMatrix, stateVals    , this.prevGramMatState);
      result = result + this.algorithm.CalcLogProbability(prevObsVals  , this.gridData, in.gridDimForGramMatrix, obsVals      , this.prevGramMatObs  );
      result = result - this.algorithm.CalcLogProbability(stateVals    , this.gridData, in.gridDimForGramMatrix, prevStateVals, gramMatState         );
      result = result - this.algorithm.CalcLogProbability(obsVals      , this.gridData, in.gridDimForGramMatrix, prevObsVals  , gramMatObs           );
    end

  end

end
