%
% Kitagawa system
%
% a1 = 0.5, b1 = 28, c1 =  8
% a2 = 0.6, b2 = 30, c2 = 10
% d1 = 0.05, d2 = 0.06, d3 = 0.07, d4 = 0.08

function [x, y] = KitagawaModel(T, a1, b1, c1, a2, b2, c2, d1, d2, d3, d4, q, r)

  q1 = q;
  q2 = q;
  r1 = r;
  r2 = r;
  r3 = r;

  x1 = zeros(1, T);
  x2 = zeros(1, T);
  y1 = zeros(1, T);
  y2 = zeros(1, T);
  y3 = zeros(1, T);

  x1(1) = 0;
  x2(1) = 0;

  for t = 1:T

    % Latent variables
    if t ~= 1
      x1(t) = a1 * x1(t-1) + b1 * x1(t-1) / (1 + x1(t-1) ^ 2) + c1 * cos(1.2 * t) + sqrt(q1) * randn;
      x2(t) = a2 * x2(t-1) + b2 * x2(t-1) / (1 + x2(t-1) ^ 2) + c2 * cos(1.2 * t) + sqrt(q2) * randn;
    end

    % Observations
    y1(t) = d1 * x1(t) ^ 2 + d2 * x2(t) ^ 2 + sqrt(r1) * randn;
    y2(t) = d3 * x1(t) ^ 2                  + sqrt(r2) * randn;
    y3(t) =                  d4 * x2(t)     + sqrt(r3) * randn;

  end

  x = [x1' x2'];
  y = [y1' y2' y3'];

end
