%
% Linear state space model
%
% http://ja.wikipedia.org/wiki/状態空間_(制御理論)
%
% k1   = 0.1;
% k2   = 0.1;
% mass = 1.0;
% q    = 0.1;
% r    = 0.1;
% dt   = 1.0;

function [x, y, F, H, Q, R] = ...
    LinearStateSpaceModel(T, k1, k2, mass, q, r, dt)

  F = [1 dt; (-k2 / mass * dt) (1 - k1 / mass * dt)];
  H = [1 0];

  Q = cov([(dt * dt) / 2; dt] * q);
  R = r * r;

  x1(1) = 1;
  x2(1) = 0;

  for t = 1:T

    % Latent variables
    if t ~= 1
      xn = F * [x1(t - 1); x2(t - 1)] + mvnrnd(0, Q);
      x1(t) = xn(1);
      x2(t) = xn(2);
    end

    % Observations
    y1(t) = H * [x1(t); x2(t)] + mvnrnd(0, R);
    
  end

  x = [x1' x2'];
  y = [y1'];

end
