%
% LinearStateSpaceModelPMMH2Estimation.m
%

rng('shuffle');

%
% Generate state and observation time series from Linear state space model
%

T    = 1000;
k1   = 0.1;
k2   = 0.1;
mass = 1.0;
q    = 0.1;
r    = 0.1;
dt   = 1.0;

[x, y, F, H] = LinearStateSpaceModel(T, k1, k2, mass, q, r, dt);

save('logs/LinearStateSpaceModel_Data.mat');

grids = { ...
  [-2:0.1:2], ...
  [-2:0.1:2]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) x(1,:)  ...
};

x0 = zeros(1, 2);
N  = 500;
J  = 100000;

gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

aspect = Aspect();
aspect.LogFileName = 'logs/LinearStateSpaceModelPMMH2.log';
aspect.MatFileNamePrefix = 'logs/LinearStateSpaceModelPMMH2';
aspect.SavesIntermediateMat = true;

disp('PMMH2 Estimation using SPAPI');

%%% Choice 1
% kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice1;

%%% Choice 2
kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;

%%% Choice 1
% mcdcSrategy = @MCDCStrategyChoice1;

%%% Choice 2
mcdcStrategy = @MCDCStrategyChoice2;

%%% Choice 3: build gram matrices here.
% for i = 1:size(stateMeanFuncs, 2)
%   stateSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% for i = 1:size(obsMeanFuncs, 2)
%   obsSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% mcdcStrategy = MCDCStrategyChoice3.Prepare(stateSigma, obsSigma);

splineHandle = @GenericSpline;
[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  PMMH2(kernelGeneratorStrategy, mcdcStrategy), ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  splineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/LinearStateSpaceModelPMMH2.mat');

disp('Done');
