%
% Lorenz dynamical system
%
% Sigma  = 10;
% Rho    = 28;
% Beta   = 8/3;
% Alpha0 = 1
% q      = 1e-1;
% r      = 1e-1;
% dt     = 1e-2;

function [x, y] = LorenzModel(T, Sigma, Rho, Beta, Alpha0, q, r, dt)

  q1 = q;
  q2 = q;
  r1 = r;
  r2 = r;
  r3 = r;

  x1 = zeros(1, T); % x
  x2 = zeros(1, T); % y
  y1 = zeros(1, T); % z1
  y2 = zeros(1, T); % z2
  y3 = zeros(1, T); % z3

  x1(1) = 0;
  x2(1) = 0;
  y1(1) = 0;
  y2(1) = 0;
  y3(1) = 0;

  for t = 2:T

    % Latent variables
    x1(t) = (Sigma * x2(t-1) * dt + x1(t-1) * (1 - Sigma * dt))   + sqrt(q1) * randn;
    x2(t) = (x1(t-1) * (Rho - y1(t-1))) * dt + x2(t-1) * (1 - dt) + sqrt(q2) * randn;

    % Observations
    y1(t) = (x1(t-1) * x2(t-1) * dt) + y1(t-1) * (1 - Beta * dt)  + sqrt(r1) * randn;
    y2(t) = Alpha0 * x1(t-1) .^ 2                                 + sqrt(r2) * randn;
    y3(t) = Alpha0 * x2(t-1) .^ 2                                 + sqrt(r3) * randn;

  end

  x = [x1' x2'];
  y = [y1' y2' y3'];

end
