%
% LorenzModelPMMH2EstimationWithoutAnchor.m
%

rng('shuffle');

%
% Generate state and observation time series from Lorenz model
%

T      = 1000;
Sigma  = 10;
Rho    = 28;
Beta   = 8/3;
Alpha0 = 1;
q      = 1e-1;
r      = 1e-1;
dt     = 1e-2;

[x, y] = LorenzModel(T, Sigma, Rho, Beta, Alpha0, q, r, dt);

save('logs/LorenzModelPMMH2WOA_Data.mat');

grids = { ...
  [-30:2:30], ...
  [-30:2:30]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2)))  ...
};

x0 = zeros(1, 2);
N  = 500;
J  = 2000;

gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

aspect = Aspect();
aspect.LogFileName = 'logs/LorenzModelPMMH2WOA.log';
aspect.MatFileNamePrefix = 'logs/LorenzModelPMMH2WOA';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 100;

disp('PMMH2 Estimation using SPAPI');

%%% Choice 1
% kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice1;

%%% Choice 2
kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;

%%% Choice 1
% mcdcSrategy = @MCDCStrategyChoice1;

%%% Choice 2
mcdcStrategy = @MCDCStrategyChoice2;

%%% Choice 3: build gram matrices here.
% for i = 1:size(stateMeanFuncs, 2)
%   stateSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% for i = 1:size(obsMeanFuncs, 2)
%   obsSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% mcdcStrategy = MCDCStrategyChoice3.Prepare(stateSigma, obsSigma);

splineHandle = @GenericSpline;
[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  PMMH2(kernelGeneratorStrategy, mcdcStrategy), ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  splineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  1, ...
  aspect ...
);

save('logs/LorenzModelPMMH2.mat');

disp('Done');
