%
% KalmanFilter.m
%

% x(t) = F * x(t-1) + B * u(t-1) + w(t)   where w(t) ~ N(0, Q)
% y(t) = H * x(t)                + v(t)   where v(t) ~ N(0, R)
%
% x1 = initial state
% P1 = initial error covariance
% u  = controls
% z  = observations

function [x, y] = KalmanFilter(F, B, H, Q, R, x1, P1, u, z)

  xdim = size(x1, 1);
  ydim = size(z, 1);
  tmax = size(z, 2);

  x = nan(xdim, tmax);
  y = nan(ydim, tmax);
  x(:, 1) = x1;
  P = P1;

  for t = 2:tmax

    % Predict
    x(:, t) = F * x(:, t-1) + B * u(:, t-1);
    P = F * P * F' + Q;

    % Update
    y(:, t) = H * x(:, t);
    r(:, t) = z(:, t) - y(:, t);
    S = H * P * H' + R;
    K = P * H' * inv(S);
    x(:, t) = x(:, t) + K * r(:, t);
    P = (eye(xdim) - K * H) * P;

  end

end
