%
% MotionCaptureTest_PMMH.m
%

rng('shuffle');

%
% データ系列の生成
%
% CMU Motion capture から 07_01 (walk) を用いる
% 推定対象は root の位置 (x, y, z) とする
%

MC_Data = amc_to_matrix('07_01.amc');
y = MC_Data(:, 1:3);

%
% PMMH による推定実験
%

initialXGrids = { ...
  [-10:5:10], ...
  [-10:5:10], ...
  [-10:5:10] ...
};

initialStateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

initialObsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

initialStateMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))) ...
};

initialObsMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))) ...
};

x0 = zeros(1, 3);
N  = 5000;
J  = 1000;

aspect = Aspect();
aspect.LogFileName = 'test.log';
aspect.MatFileNamePrefix = 'test';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 10;

disp('PMMH Estimation using SPAPI');

splineHandle = @Spline3d;
[ SKP, OKP, FV, GV, XE, YE, loglik ] = PMMH( ...
  initialXGrids, ...
  initialStateKernelGens, ...
  initialObsKernelGens, ...
  initialStateMeanFuncs, ...
  initialObsMeanFuncs, ...
  @GenericSpline, ...
  x0, ...
  y', ...
  N, ...
  J, ...
  aspect ...
);

save('MotionCaptureTest_PMMH.mat');

disp('Done');
